@@ -2,7 +2,7 @@
 """
 This module defines the main class that holds a DocTest example
 """
-from __future__ import print_function, division, absolute_import, unicode_literals
+from __future__ import absolute_import, division, print_function, unicode_literals
 import __future__
 from collections import OrderedDict
 import traceback
@@ -19,6 +19,11 @@ from xdoctest import parser
 from xdoctest import checker
 from xdoctest import exceptions
 
+from distutils.version import LooseVersion
+
+
+EVAL_MIGHT_RETURN_COROUTINE = LooseVersion(sys.version.split(' ')[0]) >= LooseVersion('3.9.0')
+
 
 class Config(dict):
     """
@@ -76,7 +81,7 @@ class Config(dict):
                                  help=('Disable ANSI coloration in stdout'))),
             (['--offset'], dict(dest='offset_linenos', action='store_true',
                                 default=self['offset_linenos'],
-                                help=('if True formated source linenumbers will agree with '
+                                help=('if True formatted source linenumbers will agree with '
                                       'their location in the source file. Otherwise they '
                                       'will be relative to the doctest itself.'))),
             (['--report'], dict(dest='reportchoice',
@@ -91,6 +96,11 @@ class Config(dict):
                                      help='exec these lines before every test')),
             (['--verbose'], dict(type=int, default=defaults.get('verbose', 3), dest='verbose',
                                      help='verbosity level')),
+            # (['--verbose'], dict(action='store_true', dest='verbose')),
+            (['--quiet'], dict(action='store_true', dest='verbose',
+                                help='sets verbosity to 1')),
+            (['--silent'], dict(action='store_false', dest='verbose',
+                                help='sets verbosity to 0')),
         ]
 
         if prefix is None:
@@ -525,8 +535,20 @@ class DocTest(object):
                             # a doctest part has `compile_mode=eval` we
                             # exepect it to return an object with a repr that
                             # can compared to a "want" statement.
+                            # print('part.compile_mode = {!r}'.format(part.compile_mode))
                             if part.compile_mode == 'eval':
+                                # print('test_globals = {}'.format(sorted(test_globals.keys())))
                                 got_eval = eval(code, test_globals)
+                                if EVAL_MIGHT_RETURN_COROUTINE:
+                                    import types
+                                    if isinstance(got_eval, types.CoroutineType):
+                                        # In 3.9-rc (2020-mar-31) it looks like
+                                        # eval sometimes returns coroutines. I
+                                        # found no docs on this. Not sure if it
+                                        # will be mainlined, but this seems to
+                                        # fix it.
+                                        import asyncio
+                                        got_eval =  asyncio.run(got_eval)
                             else:
                                 exec(code, test_globals)
 
@@ -534,7 +556,7 @@ class DocTest(object):
                         # this doctest_part.
                         self.logged_evals[partx] = got_eval
                         self.logged_stdout[partx] = cap.text
-                    except Exception as ex:
+                    except Exception:
                         if part.want:
                             # A failure may be expected if the traceback
                             # matches the part's want statement.
@@ -587,7 +609,7 @@ class DocTest(object):
                 except Exception as _ex_dbg:
                     ex_type, ex_value, tb = sys.exc_info()
 
-                    DEBUG = 1
+                    DEBUG = 0
                     if DEBUG:
                         print('_ex_dbg = {!r}'.format(_ex_dbg))
                         print('<DEBUG: doctest encountered exception>', file=sys.stderr)
