@@ -3,6 +3,7 @@
 The core logic that allows for xdoctest to parse source statically
 """
 from __future__ import absolute_import, division, print_function, unicode_literals
+import sys
 from os.path import exists
 from os.path import isfile
 from os.path import join
@@ -20,6 +21,12 @@ from xdoctest.utils.util_import import (  # NOQA
     split_modpath, modname_to_modpath, is_modname_importable,
     modpath_to_modname)
 
+import platform
+PLAT_IMPL = platform.python_implementation()
+
+
+HAS_UPDATED_LINENOS = sys.version_info[0] >= 3 and sys.version_info[1] >= 8
+
 
 class CallDefNode(object):
     """
@@ -214,19 +221,153 @@ class TopLevelVisitor(ast.NodeVisitor):
     # -- helpers ---
 
     def _docnode_line_workaround(self, docnode):
-        # lineno points to the last line of a string
-        endpos = docnode.lineno - 1
+        """
+        Find the start and ending line numbers of a docstring
+
+        CommandLine:
+            xdoctest -m xdoctest.static_analysis TopLevelVisitor._docnode_line_workaround
+
+        Example:
+            >>> from xdoctest.static_analysis import *  # NOQA
+            >>> sq = chr(39)  # single quote
+            >>> dq = chr(34)  # double quote
+            >>> source = utils.codeblock(
+                '''
+                def func0():
+                    {ddd} docstr0 {ddd}
+                def func1():
+                    {ddd}
+                    docstr1 {ddd}
+                def func2():
+                    {ddd} docstr2
+                    {ddd}
+                def func3():
+                    {ddd}
+                    docstr3
+                    {ddd}  # foobar
+                def func5():
+                    {ddd}pathological case
+                    {sss} # {ddd} # {sss} # {ddd} # {ddd}
+                def func6():
+                    " single quoted docstr "
+                def func7():
+                    r{ddd}
+                    raw line
+                    {ddd}
+                ''').format(ddd=dq * 3, sss=sq * 3)
+            >>> self = TopLevelVisitor(source)
+            >>> func_nodes = self.syntax_tree().body
+            >>> print(utils.add_line_numbers(utils.highlight_code(source), start=1))
+            >>> wants = [
+            >>>     (2, 2),
+            >>>     (4, 5),
+            >>>     (7, 8),
+            >>>     (10, 12),
+            >>>     (14, 15),
+            >>>     (17, 17),
+            >>>     (19, 21),
+            >>> ]
+            >>> for i, func_node in enumerate(func_nodes):
+            >>>     docnode = func_node.body[0]
+            >>>     got = self._docnode_line_workaround(docnode)
+            >>>     want = wants[i]
+            >>>     print('got = {!r}'.format(got))
+            >>>     print('want = {!r}'.format(want))
+            >>>     assert got == want
+        """
+        # lineno points to the last line of a string in CPython < 3.8
+        if hasattr(docnode, 'end_lineno'):
+            endpos = docnode.end_lineno - 1
+        else:
+            if PLAT_IMPL == 'PyPy':
+                startpos = docnode.lineno - 1
+                docstr = utils.ensure_unicode(docnode.value.s)
+                sourcelines = self.sourcelines
+                start, stop = self._find_docstr_endpos_workaround(docstr,
+                                                                  sourcelines,
+                                                                  startpos)
+                # Convert 0-based line positions to 1-based line numbers
+                doclineno = start + 1
+                doclineno_end = stop + 1
+                return doclineno, doclineno_end
+            else:
+                # Hack for older versions
+                # TODO: fix in pypy
+                endpos = docnode.lineno - 1
+
         docstr = utils.ensure_unicode(docnode.value.s)
         sourcelines = self.sourcelines
-        start, stop = self._docstr_line_workaround(docstr, sourcelines, endpos)
+        start, stop = self._find_docstr_startpos_workaround(docstr, sourcelines, endpos)
         # Convert 0-based line positions to 1-based line numbers
         doclineno = start + 1
         doclineno_end = stop
         # print('docnode = {!r}'.format(docnode))
         return doclineno, doclineno_end
 
-    def _docstr_line_workaround(self, docstr, sourcelines, endpos):
+    @classmethod
+    def _find_docstr_endpos_workaround(cls, docstr, sourcelines, startpos):
+        """
+        Like docstr_line_workaround, but works from the top-down instead of
+        bottom-up. This is for pypy.
+
+
+        Given a docstring, its original source lines, and where the start
+        position is, this function finds the end-position of the docstr
+
+        Example:
+            >>> fmtkw = dict(sss=chr(39) * 3, ddd=chr(34) * 3)
+            >>> source = utils.codeblock(
+                    '''
+                    {ddd}
+                    docstr0
+                    {ddd}
+                    '''.format(**fmtkw))
+            >>> sourcelines = source.splitlines()
+            >>> docstr = eval(source, {}, {})
+            >>> startpos = 0
+            >>> start, stop = TopLevelVisitor._find_docstr_endpos_workaround(docstr, sourcelines, startpos)
+            >>> assert (start, stop) == (0, 2)
+            >>> #
+            >>> source = utils.codeblock(
+                    '''
+                    "docstr0"
+                    '''.format(**fmtkw))
+            >>> sourcelines = source.splitlines()
+            >>> docstr = eval(source, {}, {})
+            >>> startpos = 0
+            >>> start, stop = TopLevelVisitor._find_docstr_endpos_workaround(docstr, sourcelines, startpos)
+            >>> assert (start, stop) == (0, 0)
+        """
+        start = startpos
+        stop = startpos
+        startline = sourcelines[start]
+
+        trips = ("'''", '"""')
+        for trip in trips:
+            if startline.strip().startswith((trip, 'r' + trip)):
+                nlines = docstr.count('\n')
+                # assuming that the docstr is actually terminated with this
+                # kind of triple quote, then the end line is at this position
+                cand_stop_ = start + nlines
+                endline = sourcelines[cand_stop_]
+
+                endpat = re.escape(trip) + r'\s*#.*$'
+                endline_ = re.sub(endpat, trip, endline).strip()
+
+                # The startline should also begin with the same triple quote
+                # Account for raw strings. Note f-strings cannot be docstrings
+                if endline_.endswith(trip):
+                    stop = cand_stop_
+                    break
+                else:
+                    # Conditions failed, revert to assuming a one-line string.
+                    stop = start
+        return start, stop
+
+    def _find_docstr_startpos_workaround(self, docstr, sourcelines, endpos):
         r"""
+        Find the which sourcelines contain the docstring
+
         Args:
             docstr (str): the extracted docstring.
 
@@ -245,11 +386,16 @@ class TopLevelVisitor(ast.NodeVisitor):
                 stop: the line position (0 based) that the docstring stops
 
                 such that sourcelines[start:stop] will contain the docstring
+
         CommandLine:
-            python -m xdoctest.static_analysis TopLevelVisitor._docstr_line_workaround
+            python -m xdoctest xdoctest.static_analysis TopLevelVisitor._find_docstr_startpos_workaround
+            python -m xdoctest xdoctest.static_analysis TopLevelVisitor._find_docstr_startpos_workaround --debug
 
         Example:
+            >>> # xdoctest: +REQUIRES(CPython)
+            >>> # This function is a specific workaround for a CPython bug.
             >>> from xdoctest.static_analysis import *
+            >>> sys.DEBUG = '--debug' in sys.argv
             >>> sq = chr(39)  # single quote
             >>> dq = chr(34)  # double quote
             >>> source = utils.codeblock(
@@ -288,23 +434,59 @@ class TopLevelVisitor(ast.NodeVisitor):
             >>> ]
             >>> self = TopLevelVisitor.parse(source)
             >>> pt = ast.parse(source.encode('utf8'))
+            >>> sourcelines = source.splitlines()
+            >>> # PYPY docnode.lineno specify the startpos of a docstring not
+            >>> # the end.
             >>> print('\n\n====\n\n')
+            >>> #for i in [0, 1]:
             >>> for i in range(len(targets)):
             >>>     print('----------')
-            >>>     sourcelines = source.splitlines()
             >>>     funcnode = pt.body[i]
+            >>>     print('funcnode = {!r}'.format(funcnode))
             >>>     docnode = funcnode.body[0]
+            >>>     print('funcnode.__dict__ = {!r}'.format(funcnode.__dict__))
+            >>>     print('docnode = {!r}'.format(docnode))
+            >>>     print('docnode.value = {!r}'.format(docnode.value))
+            >>>     print('docnode.value.__dict__ = {!r}'.format(docnode.value.__dict__))
+            >>>     print('docnode.value.s = {!r}'.format(docnode.value.s))
+            >>>     print('docnode.lineno = {!r}'.format(docnode.lineno))
+            >>>     print('docnode.col_offset = {!r}'.format(docnode.col_offset))
+            >>>     print('docnode = {!r}'.format(docnode))
+            >>>     #import IPython
+            >>>     #IPython.embed()
             >>>     docstr = ast.get_docstring(funcnode, clean=False)
+            >>>     print('len(docstr) = {}'.format(len(docstr)))
             >>>     endpos = docnode.lineno - 1
-            >>>     start, end = self._docstr_line_workaround(docstr, sourcelines, endpos)
+            >>>     if hasattr(docnode, 'end_lineno'):
+            >>>         endpos = docnode.end_lineno - 1
+            >>>     print('endpos = {!r}'.format(endpos))
+            >>>     start, end = self._find_docstr_startpos_workaround(docstr, sourcelines, endpos)
+            >>>     print('i = {!r}'.format(i))
             >>>     print('got  = {}, {}'.format(start, end))
             >>>     print('want = {}, {}'.format(*targets[i]))
-            >>>     assert targets[i] == (start, end)
+            >>>     if targets[i] != (start, end):
+            >>>         print('---')
+            >>>         print(docstr)
+            >>>         print('---')
+            >>>         print('sourcelines = [\n{}\n]'.format(', \n'.join(list(map(repr, enumerate(sourcelines))))))
+            >>>         print('endpos = {!r}'.format(endpos))
+            >>>         raise AssertionError('docstr workaround is failing')
             >>>     print('----------')
+            >>> sys.DEBUG = 0
         """
         # First assume a one-line string that starts and stops on the same line
         start = endpos
         stop = endpos + 1
+        endline = sourcelines[stop - 1]
+
+        DEBUG = getattr(sys, 'DEBUG', 0)
+        if DEBUG:
+            print('----<<<')
+            #print('sourcelines = [{}]'.format('\n'.join(list(map(repr, sourcelines)))))
+            print('endpos = {!r}'.format(endpos))
+            print('start = {!r}'.format(start))
+            print('stop = {!r}'.format(stop))
+            print('endline = {!r}'.format(endline))
 
         # Determine if the docstring is a triple quoted string, by trying both
         # triple quote styles and checking if the string starts and ends with
@@ -312,8 +494,9 @@ class TopLevelVisitor(ast.NodeVisitor):
         # quoted string literal and can therefore safely extract the starting
         # line position.
         trips = ("'''", '"""')
-        endline = sourcelines[stop - 1]
         for trip in trips:
+            if DEBUG:
+                print('trip = {!r}'.format(trip))
             pattern = re.escape(trip) + r'\s*#.*$'
             # Assuming the multiline string is using `trip` as the triple quote
             # format, then the first instance of that pattern must terminate
@@ -322,14 +505,19 @@ class TopLevelVisitor(ast.NodeVisitor):
             # ignored. The above pattern will match the first triple quote it
             # sees, and then will remove any trailing comments.
             endline_ = re.sub(pattern, trip, endline).strip()
-            # After removing commends, if the endline endswith a triple quote,
+            # After removing comments, if the endline endswith a triple quote,
             # then we must be in a multiline string IF the startline starts
             # with that same triple quote. We should be able to determine where
             # the startline is because we know how many newline characters are
             # in the extracted docstring. This works because all newline
             # characters in multiline string literals MUST correspond to actual
             # newlines in the source code.
+            if DEBUG:
+                print('pattern = {!r}'.format(pattern))
+                print('endline_ = {!r}'.format(endline_))
             if endline_.endswith(trip):
+                if DEBUG:
+                    print('endline ended with trip')
                 nlines = docstr.count('\n')
                 # assuming that the docstr is actually terminated with this
                 # kind of triple quote, then the start line is at this position
@@ -340,16 +528,30 @@ class TopLevelVisitor(ast.NodeVisitor):
                 # Account for raw strings. Note f-strings cannot be docstrings
                 if startline.strip().startswith((trip, 'r' + trip)):
                     # Both conditions pass.
+                    if DEBUG:
+                        print('startline did end with trip')
                     start = cand_start_
                     break
                 else:
                     # Conditions failed, revert to assuming a one-line string.
+                    if DEBUG:
+                        print('startline did not end with trip')
                     start = stop - 1
+            else:
+                if DEBUG:
+                    print('endline did not end with trip')
 
+        if DEBUG:
+            print('start = {!r}'.format(start))
+            print('stop = {!r}'.format(stop))
+            print('----<<<')
         return start, stop
 
     def _get_docstring(self, node):
         """
+        CommandLine:
+            xdoctest -m ~/code/xdoctest/xdoctest/static_analysis.py TopLevelVisitor._get_docstring
+
         Example:
             >>> source = utils.codeblock(
                 '''
@@ -372,6 +574,9 @@ class TopLevelVisitor(ast.NodeVisitor):
 
     def _workaround_func_lineno(self, node):
         """
+        Finds the correct line for the original function definition even when
+        decorators are involved.
+
         Example:
             >>> source = utils.codeblock(
                 '''
@@ -424,7 +629,7 @@ def parse_calldefs(source=None, fpath=None):
         try:
             with open(fpath, 'rb') as file_:
                 source = file_.read().decode('utf-8')
-        except Exception as ex:
+        except Exception:
             try:
                 with open(fpath, 'rb') as file_:
                     source = file_.read()
@@ -446,6 +651,7 @@ def _parse_static_node_value(node):
     """
     Extract a constant value from a node if possible
     """
+    # TODO: ast.Constant for 3.8
     if isinstance(node, ast.Num):
         value = node.n
     elif isinstance(node, ast.Str):
@@ -506,7 +712,7 @@ def parse_static_value(key, source=None, fpath=None):
         try:
             with open(fpath, 'rb') as file_:
                 source = file_.read().decode('utf-8')
-        except Exception as ex:
+        except Exception:
             with open(fpath, 'rb') as file_:
                 source = file_.read()
     pt = ast.parse(source)
@@ -723,7 +929,7 @@ def extract_comments(source):
         for t in tokenize.generate_tokens(_readline):
             if t[0] == tokenize.COMMENT:
                 yield t[1]
-    except tokenize.TokenError as ex:
+    except tokenize.TokenError:
         pass
 
 
